// Definicije metoda i prijateljskih funkcija uz klasu Poli.

#include "poli.h"
#include <stdlib.h>

void Poli::greska (Greska g) const {       // Obrada greske.
  const char *poruke[] = { "Neuspesna dodela memorije polinomu",
                           "Nedozvoljen red polinoma",
                           "Nedozvoljen indeks koeficijenta polinoma"
                         };
  cout << "*** " << poruke[g] << "! ***\n";
  exit (g);
}

void Poli::kopiraj (const double *aa, int nn){ // Kopiranje u tekuci obj.
  if ((n = nn) < -1)                greska (G_RED);
  if ((a = new double [nn+1]) == 0) greska (G_MEM);
  for (int i=0; i<=nn; i++) a[i] = aa[i];
}

void Poli::niz (double *aa, int &nn) const   // Smestanje polinoma u niz.
  { for (int i=0; i<=n; i++) aa[i] = a[i]; nn = n; }

double & Poli::operator[] (int ind) const {  // Dohvatanje koeficijenta.
  if (ind < 0 || ind > n) greska (G_IND);
  return a[ind];
}

double Poli::operator() (double x) const    // Vrednost polinoma.
  { double s = 0; for (int i=n; i>=0; (s*=x)+=a[i--]); return s; }

Poli operator+ (const Poli &p1, const Poli &p2){ // Zbir dva polinoma.
  int n;
  if (p1.n == p2.n) for (n=p1.n; n>=0 && p1.a[n]+p2.a[n]==0; n--);
    else            n = (p1.n > p2.n) ? p1.n : p2.n;
  Poli p; p.n = n;
  if ((p.a = new double [n+1]) == 0) p.greska (Poli::G_MEM);
  for (int i=0; i<=n; i++)
    if      (i > p2.n)  p.a[i] = p1.a[i];
    else if (i > p1.n)  p.a[i] =           p2.a[i];
    else                p.a[i] = p1.a[i] + p2.a[i];
  return p;
}

Poli operator- (const Poli &p1, const Poli &p2){ // Razlika dva polinoma.
  int n;
  if (p1.n == p2.n) for (n=p1.n; n>=0 && p1.a[n]-p2.a[n]==0; n--);
    else            n = (p1.n > p2.n) ? p1.n : p2.n;
  Poli p; p.n = n;
  if ((p.a = new double [n+1]) == 0) p.greska (Poli::G_MEM);
  for (int i=0; i<=n; i++)
    if      (i > p2.n)  p.a[i] = p1.a[i];
    else if (i > p1.n)  p.a[i] =         - p2.a[i];
    else                p.a[i] = p1.a[i] - p2.a[i];
  return p;
}

Poli operator* (const Poli &p1, const Poli &p2){ // Proizvod dva polinoma
  int n = (p1.n>=0 && p2.n>=0) ? p1.n+p2.n : -1;
  Poli p; p.n = n;
  if ((p.a = new double [n+1]) == 0) p.greska (Poli::G_MEM);
  for (int i=0; i<=n; p.a[i++]=0);
  for (i=0; i<=p1.n; i++)
    for (int j=0; j<=p2.n; j++)
      p.a[i+j] += p1.a[i] * p2.a[j];
  return p;
}

istream & operator>> (istream &d, Poli &p) {       // Citanje polinoma.
  Poli q; d >> q.n;      if (q.n < -1) q.greska (Poli::G_RED);
  if ((q.a = new double [q.n+1]) == 0) q.greska (Poli::G_MEM);
  for (int i=q.n; i>=0; d >> q.a[i--]);
  p = q; return d;
}

ostream & operator<< (ostream &d, const Poli &p) { // Pisanje polinoma.
  d << "p[";
  for (int i=p.n; i>=0; i--) d << p.a[i] << (i ? "," : "");
  d << ']';
  return d;
}
